@@ -1,9 +1,10 @@
 /*
- * gedit-print.c
+ * gedit-print-job.c
  * This file is part of gedit
  *
  * Copyright (C) 2000-2001 Chema Celorio, Paolo Maggi
  * Copyright (C) 2002-2008 Paolo Maggi
+ * Copyright (C) 2015 Sébastien Wilmet
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,42 +38,33 @@
 
 struct _GeditPrintJobPrivate
 {
-	GSettings                *print_settings;
+	GSettings *gsettings;
 
-	GeditView                *view;
-	GeditDocument            *doc;
+	GeditView *view;
+	GeditDocument *doc;
 
-	GtkPrintOperation        *operation;
+	GtkPrintOperation *operation;
 	GtkSourcePrintCompositor *compositor;
 
-	GtkPrintSettings         *settings;
+	GtkWidget *preview;
 
-	GtkWidget                *preview;
+	gchar *status_string;
+	gdouble progress;
 
-	GeditPrintJobStatus       status;
+	/* Widgets part of the custom print preferences widget.
+	 * These pointers are valid just when the dialog is displayed.
+	 */
+	GtkToggleButton *syntax_checkbutton;
+	GtkToggleButton *page_header_checkbutton;
+	GtkToggleButton *line_numbers_checkbutton;
+	GtkSpinButton *line_numbers_spinbutton;
+	GtkToggleButton *text_wrapping_checkbutton;
+	GtkToggleButton *do_not_split_checkbutton;
+	GtkFontButton *body_fontbutton;
+	GtkFontButton *headers_fontbutton;
+	GtkFontButton *numbers_fontbutton;
 
-	gchar                    *status_string;
-
-	gdouble			  progress;
-
-	gboolean                  is_preview;
-
-	/* widgets part of the custom print preferences widget.
-	 * These pointers are valid just when the dialog is displayed */
-	GtkWidget *syntax_checkbutton;
-	GtkWidget *page_header_checkbutton;
-	GtkWidget *line_numbers_checkbutton;
-	GtkWidget *line_numbers_hbox;
-	GtkWidget *line_numbers_spinbutton;
-	GtkWidget *text_wrapping_checkbutton;
-	GtkWidget *do_not_split_checkbutton;
-	GtkWidget *body_font_label;
-	GtkWidget *headers_font_label;
-	GtkWidget *numbers_font_label;
-	GtkWidget *body_fontbutton;
-	GtkWidget *headers_fontbutton;
-	GtkWidget *numbers_fontbutton;
-	GtkWidget *restore_button;
+	guint is_preview : 1;
 };
 
 enum
@@ -93,16 +85,9 @@ static guint print_job_signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (GeditPrintJob, gedit_print_job, G_TYPE_OBJECT)
 
-static void          end_print_cb      (GtkPrintOperation *operation,
-                                        GtkPrintContext   *context,
-                                        GeditPrintJob     *job);
-
-static void          done_cb           (GtkPrintOperation       *operation,
-                                        GtkPrintOperationResult  result,
-                                        GeditPrintJob           *job);
-
 static void
-set_view (GeditPrintJob *job, GeditView *view)
+set_view (GeditPrintJob *job,
+	  GeditView     *view)
 {
 	job->priv->view = view;
 	job->priv->doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
@@ -121,6 +106,7 @@ gedit_print_job_get_property (GObject    *object,
 		case PROP_VIEW:
 			g_value_set_object (value, job->priv->view);
 			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -140,12 +126,26 @@ gedit_print_job_set_property (GObject      *object,
 		case PROP_VIEW:
 			set_view (job, g_value_get_object (value));
 			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
 	}
 }
 
+static void
+gedit_print_job_dispose (GObject *object)
+{
+	GeditPrintJob *job = GEDIT_PRINT_JOB (object);
+
+	g_clear_object (&job->priv->gsettings);
+	g_clear_object (&job->priv->operation);
+	g_clear_object (&job->priv->compositor);
+	g_clear_object (&job->priv->preview);
+
+	G_OBJECT_CLASS (gedit_print_job_parent_class)->dispose (object);
+}
+
 static void
 gedit_print_job_finalize (GObject *object)
 {
@@ -156,27 +156,6 @@ gedit_print_job_finalize (GObject *object)
 	G_OBJECT_CLASS (gedit_print_job_parent_class)->finalize (object);
 }
 
-static void
-gedit_print_job_dispose (GObject *object)
-{
-	GeditPrintJob *job = GEDIT_PRINT_JOB (object);
-
-	g_clear_object (&job->priv->print_settings);
-	g_clear_object (&job->priv->compositor);
-
-	if (job->priv->operation != NULL)
-	{
-		g_signal_handlers_disconnect_by_func (job->priv->operation,
-		                                      end_print_cb, job);
-		g_signal_handlers_disconnect_by_func (job->priv->operation,
-		                                      done_cb, job);
-		g_object_unref (job->priv->operation);
-		job->priv->operation = NULL;
-	}
-
-	G_OBJECT_CLASS (gedit_print_job_parent_class)->dispose (object);
-}
-
 static void
 gedit_print_job_class_init (GeditPrintJobClass *klass)
 {
@@ -186,8 +165,8 @@ gedit_print_job_class_init (GeditPrintJobClass *klass)
 
 	object_class->get_property = gedit_print_job_get_property;
 	object_class->set_property = gedit_print_job_set_property;
-	object_class->finalize = gedit_print_job_finalize;
 	object_class->dispose = gedit_print_job_dispose;
+	object_class->finalize = gedit_print_job_finalize;
 
 	g_object_class_install_property (object_class,
 					 PROP_VIEW,
@@ -235,33 +214,13 @@ gedit_print_job_class_init (GeditPrintJobClass *klass)
 }
 
 static void
-line_numbers_checkbutton_toggled (GtkToggleButton *button,
-				  GeditPrintJob   *job)
+gedit_print_job_init (GeditPrintJob *job)
 {
-	gtk_widget_set_sensitive (job->priv->line_numbers_hbox,
-				  gtk_toggle_button_get_active (button));
-}
+	job->priv = gedit_print_job_get_instance_private (job);
 
-static void
-wrap_mode_checkbutton_toggled (GtkToggleButton *button,
-			       GeditPrintJob   *job)
-{
-	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (job->priv->text_wrapping_checkbutton)))
-	{
-		gtk_widget_set_sensitive (job->priv->do_not_split_checkbutton,
-					  FALSE);
-		gtk_toggle_button_set_inconsistent (
-			GTK_TOGGLE_BUTTON (job->priv->do_not_split_checkbutton),
-					   TRUE);
-	}
-	else
-	{
-		gtk_widget_set_sensitive (job->priv->do_not_split_checkbutton,
-					  TRUE);
-		gtk_toggle_button_set_inconsistent (
-			GTK_TOGGLE_BUTTON (job->priv->do_not_split_checkbutton),
-					   FALSE);
-	}
+	job->priv->gsettings = g_settings_new ("org.gnome.gedit.preferences.print");
+
+	job->priv->status_string = g_strdup (_("Preparing..."));
 }
 
 static void
@@ -269,35 +228,9 @@ restore_button_clicked (GtkButton     *button,
 			GeditPrintJob *job)
 
 {
-	gchar *body, *header, *numbers;
-
-	g_settings_reset (job->priv->print_settings,
-			  GEDIT_SETTINGS_PRINT_FONT_BODY_PANGO);
-	g_settings_reset (job->priv->print_settings,
-			  GEDIT_SETTINGS_PRINT_FONT_HEADER_PANGO);
-	g_settings_reset (job->priv->print_settings,
-			  GEDIT_SETTINGS_PRINT_FONT_NUMBERS_PANGO);
-
-	body = g_settings_get_string (job->priv->print_settings,
-				      GEDIT_SETTINGS_PRINT_FONT_BODY_PANGO);
-	header = g_settings_get_string (job->priv->print_settings,
-					GEDIT_SETTINGS_PRINT_FONT_HEADER_PANGO);
-	numbers = g_settings_get_string (job->priv->print_settings,
-					 GEDIT_SETTINGS_PRINT_FONT_NUMBERS_PANGO);
-
-	gtk_font_button_set_font_name (
-			GTK_FONT_BUTTON (job->priv->body_fontbutton),
-			body);
-	gtk_font_button_set_font_name (
-			GTK_FONT_BUTTON (job->priv->headers_fontbutton),
-			header);
-	gtk_font_button_set_font_name (
-			GTK_FONT_BUTTON (job->priv->numbers_fontbutton),
-			numbers);
-
-	g_free (body);
-	g_free (header);
-	g_free (numbers);
+	g_settings_reset (job->priv->gsettings, GEDIT_SETTINGS_PRINT_FONT_BODY_PANGO);
+	g_settings_reset (job->priv->gsettings, GEDIT_SETTINGS_PRINT_FONT_HEADER_PANGO);
+	g_settings_reset (job->priv->gsettings, GEDIT_SETTINGS_PRINT_FONT_NUMBERS_PANGO);
 }
 
 static GObject *
@@ -306,11 +239,11 @@ create_custom_widget_cb (GtkPrintOperation *operation,
 {
 	GtkBuilder *builder;
 	GtkWidget *contents;
-	gint line_numbers;
+	GtkWidget *line_numbers_hbox;
+	GtkWidget *restore_button;
+	guint line_numbers;
 	GtkWrapMode wrap_mode;
-	gboolean syntax_hl;
-	gboolean print_header;
-	gchar *font_body, *font_header, *font_numbers;
+
 	gchar *root_objects[] = {
 		"adjustment1",
 		"contents",
@@ -322,115 +255,93 @@ create_custom_widget_cb (GtkPrintOperation *operation,
 	                                       root_objects, NULL);
 	contents = GTK_WIDGET (gtk_builder_get_object (builder, "contents"));
 	g_object_ref (contents);
-	job->priv->syntax_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "syntax_checkbutton"));
-	job->priv->line_numbers_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "line_numbers_checkbutton"));
-	job->priv->line_numbers_hbox = GTK_WIDGET (gtk_builder_get_object (builder, "line_numbers_hbox"));
-	job->priv->line_numbers_spinbutton = GTK_WIDGET (gtk_builder_get_object (builder, "line_numbers_spinbutton"));
-	job->priv->page_header_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "page_header_checkbutton"));
-	job->priv->text_wrapping_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "text_wrapping_checkbutton"));
-	job->priv->do_not_split_checkbutton = GTK_WIDGET (gtk_builder_get_object (builder, "do_not_split_checkbutton"));
-	job->priv->body_font_label = GTK_WIDGET (gtk_builder_get_object (builder, "body_font_label"));
-	job->priv->body_fontbutton = GTK_WIDGET (gtk_builder_get_object (builder, "body_fontbutton"));
-	job->priv->headers_font_label = GTK_WIDGET (gtk_builder_get_object (builder, "headers_font_label"));
-	job->priv->headers_fontbutton = GTK_WIDGET (gtk_builder_get_object (builder, "headers_fontbutton"));
-	job->priv->numbers_font_label = GTK_WIDGET (gtk_builder_get_object (builder, "numbers_font_label"));
-	job->priv->numbers_fontbutton = GTK_WIDGET (gtk_builder_get_object (builder, "numbers_fontbutton"));
-	job->priv->restore_button = GTK_WIDGET (gtk_builder_get_object (builder, "restore_button"));
+	job->priv->syntax_checkbutton = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "syntax_checkbutton"));
+	job->priv->line_numbers_checkbutton = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "line_numbers_checkbutton"));
+	line_numbers_hbox = GTK_WIDGET (gtk_builder_get_object (builder, "line_numbers_hbox"));
+	job->priv->line_numbers_spinbutton = GTK_SPIN_BUTTON (gtk_builder_get_object (builder, "line_numbers_spinbutton"));
+	job->priv->page_header_checkbutton = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "page_header_checkbutton"));
+	job->priv->text_wrapping_checkbutton = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "text_wrapping_checkbutton"));
+	job->priv->do_not_split_checkbutton = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "do_not_split_checkbutton"));
+	job->priv->body_fontbutton = GTK_FONT_BUTTON (gtk_builder_get_object (builder, "body_fontbutton"));
+	job->priv->headers_fontbutton = GTK_FONT_BUTTON (gtk_builder_get_object (builder, "headers_fontbutton"));
+	job->priv->numbers_fontbutton = GTK_FONT_BUTTON (gtk_builder_get_object (builder, "numbers_fontbutton"));
+	restore_button = GTK_WIDGET (gtk_builder_get_object (builder, "restore_button"));
 	g_object_unref (builder);
 
-	/* Get all settings values */
-	syntax_hl = g_settings_get_boolean (job->priv->print_settings,
-					    GEDIT_SETTINGS_PRINT_SYNTAX_HIGHLIGHTING);
-	print_header = g_settings_get_boolean (job->priv->print_settings,
-					       GEDIT_SETTINGS_PRINT_HEADER);
-	g_settings_get (job->priv->print_settings, GEDIT_SETTINGS_PRINT_LINE_NUMBERS,
-			"u", &line_numbers);
-	font_body = g_settings_get_string (job->priv->print_settings,
-					   GEDIT_SETTINGS_PRINT_FONT_BODY_PANGO);
-	font_header = g_settings_get_string (job->priv->print_settings,
-					     GEDIT_SETTINGS_PRINT_FONT_HEADER_PANGO);
-	font_numbers = g_settings_get_string (job->priv->print_settings,
-					      GEDIT_SETTINGS_PRINT_FONT_NUMBERS_PANGO);
+	/* Syntax highlighting */
+	g_settings_bind (job->priv->gsettings, GEDIT_SETTINGS_PRINT_SYNTAX_HIGHLIGHTING,
+			 job->priv->syntax_checkbutton, "active",
+			 G_SETTINGS_BIND_GET);
 
-	/* Print syntax */
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (job->priv->syntax_checkbutton),
-				      syntax_hl);
-
-	/* Print page headers */
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (job->priv->page_header_checkbutton),
-				      print_header);
+	/* Print header */
+	g_settings_bind (job->priv->gsettings, GEDIT_SETTINGS_PRINT_HEADER,
+			 job->priv->page_header_checkbutton, "active",
+			 G_SETTINGS_BIND_GET);
 
 	/* Line numbers */
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (job->priv->line_numbers_checkbutton),
-				      line_numbers > 0);
+	g_settings_get (job->priv->gsettings, GEDIT_SETTINGS_PRINT_LINE_NUMBERS,
+			"u", &line_numbers);
 
 	if (line_numbers > 0)
 	{
-		gtk_spin_button_set_value (GTK_SPIN_BUTTON (job->priv->line_numbers_spinbutton),
-					   (guint) line_numbers);
-		gtk_widget_set_sensitive (job->priv->line_numbers_hbox, TRUE);
+		gtk_spin_button_set_value (job->priv->line_numbers_spinbutton, line_numbers);
 	}
 	else
 	{
-		gtk_spin_button_set_value (GTK_SPIN_BUTTON (job->priv->line_numbers_spinbutton),
-					   1);
-		gtk_widget_set_sensitive (job->priv->line_numbers_hbox, FALSE);
+		gtk_spin_button_set_value (job->priv->line_numbers_spinbutton, 1);
 	}
 
-	/* Text wrapping */
-	wrap_mode = g_settings_get_enum (job->priv->print_settings,
+	gtk_toggle_button_set_active (job->priv->line_numbers_checkbutton,
+				      line_numbers > 0);
+
+	g_object_bind_property (job->priv->line_numbers_checkbutton, "active",
+				line_numbers_hbox, "sensitive",
+				G_BINDING_SYNC_CREATE);
+
+	/* Fonts */
+	g_settings_bind (job->priv->gsettings, GEDIT_SETTINGS_PRINT_FONT_BODY_PANGO,
+			 job->priv->body_fontbutton, "font-name",
+			 G_SETTINGS_BIND_GET);
+
+	g_settings_bind (job->priv->gsettings, GEDIT_SETTINGS_PRINT_FONT_HEADER_PANGO,
+			 job->priv->headers_fontbutton, "font-name",
+			 G_SETTINGS_BIND_GET);
+
+	g_settings_bind (job->priv->gsettings, GEDIT_SETTINGS_PRINT_FONT_NUMBERS_PANGO,
+			 job->priv->numbers_fontbutton, "font-name",
+			 G_SETTINGS_BIND_GET);
+
+	/* Wrap mode */
+	wrap_mode = g_settings_get_enum (job->priv->gsettings,
 					 GEDIT_SETTINGS_PRINT_WRAP_MODE);
 
 	switch (wrap_mode)
 	{
 		case GTK_WRAP_WORD:
-			gtk_toggle_button_set_active (
-				GTK_TOGGLE_BUTTON (job->priv->text_wrapping_checkbutton), TRUE);
-			gtk_toggle_button_set_active (
-				GTK_TOGGLE_BUTTON (job->priv->do_not_split_checkbutton), TRUE);
+			gtk_toggle_button_set_active (job->priv->text_wrapping_checkbutton, TRUE);
+			gtk_toggle_button_set_active (job->priv->do_not_split_checkbutton, TRUE);
 			break;
+
 		case GTK_WRAP_CHAR:
-			gtk_toggle_button_set_active (
-				GTK_TOGGLE_BUTTON (job->priv->text_wrapping_checkbutton), TRUE);
-			gtk_toggle_button_set_active (
-				GTK_TOGGLE_BUTTON (job->priv->do_not_split_checkbutton), FALSE);
+			gtk_toggle_button_set_active (job->priv->text_wrapping_checkbutton, TRUE);
+			gtk_toggle_button_set_active (job->priv->do_not_split_checkbutton, FALSE);
 			break;
+
 		default:
-			gtk_toggle_button_set_active (
-				GTK_TOGGLE_BUTTON (job->priv->text_wrapping_checkbutton), FALSE);
-			gtk_toggle_button_set_inconsistent (
-				GTK_TOGGLE_BUTTON (job->priv->do_not_split_checkbutton), TRUE);
+			gtk_toggle_button_set_active (job->priv->text_wrapping_checkbutton, FALSE);
+			break;
 	}
 
-	gtk_widget_set_sensitive (job->priv->do_not_split_checkbutton,
-				  wrap_mode != GTK_WRAP_NONE);
+	g_object_bind_property (job->priv->text_wrapping_checkbutton, "active",
+				job->priv->do_not_split_checkbutton, "sensitive",
+				G_BINDING_SYNC_CREATE);
 
-	/* Set initial values */
-	gtk_font_button_set_font_name (GTK_FONT_BUTTON (job->priv->body_fontbutton),
-				       font_body);
-	g_free (font_body);
+	g_object_bind_property (job->priv->text_wrapping_checkbutton, "active",
+				job->priv->do_not_split_checkbutton, "inconsistent",
+				G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE);
 
-	gtk_font_button_set_font_name (GTK_FONT_BUTTON (job->priv->headers_fontbutton),
-				       font_header);
-	g_free (font_header);
-
-	gtk_font_button_set_font_name (GTK_FONT_BUTTON (job->priv->numbers_fontbutton),
-				       font_numbers);
-	g_free (font_numbers);
-
-	g_signal_connect (job->priv->line_numbers_checkbutton,
-			  "toggled",
-			  G_CALLBACK (line_numbers_checkbutton_toggled),
-			  job);
-	g_signal_connect (job->priv->text_wrapping_checkbutton,
-			  "toggled",
-			  G_CALLBACK (wrap_mode_checkbutton_toggled),
-			  job);
-	g_signal_connect (job->priv->do_not_split_checkbutton,
-			  "toggled",
-			  G_CALLBACK (wrap_mode_checkbutton_toggled),
-			  job);
-	g_signal_connect (job->priv->restore_button,
+	/* Restore */
+	g_signal_connect (restore_button,
 			  "clicked",
 			  G_CALLBACK (restore_button_clicked),
 			  job);
@@ -443,46 +354,59 @@ custom_widget_apply_cb (GtkPrintOperation *operation,
 			GtkWidget         *widget,
 			GeditPrintJob     *job)
 {
-	gboolean syntax, page_header;
-	const gchar *body, *header, *numbers;
+	gboolean syntax;
+	gboolean page_header;
+	const gchar *body_font;
+	const gchar *header_font;
+	const gchar *numbers_font;
 	GtkWrapMode wrap_mode;
 
-	syntax = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (job->priv->syntax_checkbutton));
-	page_header = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (job->priv->page_header_checkbutton));
-	body = gtk_font_button_get_font_name (GTK_FONT_BUTTON (job->priv->body_fontbutton));
-	header = gtk_font_button_get_font_name (GTK_FONT_BUTTON (job->priv->headers_fontbutton));
-	numbers = gtk_font_button_get_font_name (GTK_FONT_BUTTON (job->priv->numbers_fontbutton));
+	syntax = gtk_toggle_button_get_active (job->priv->syntax_checkbutton);
+	page_header = gtk_toggle_button_get_active (job->priv->page_header_checkbutton);
+	body_font = gtk_font_button_get_font_name (job->priv->body_fontbutton);
+	header_font = gtk_font_button_get_font_name (job->priv->headers_fontbutton);
+	numbers_font = gtk_font_button_get_font_name (job->priv->numbers_fontbutton);
 
-	g_settings_set_boolean (job->priv->print_settings,
-				GEDIT_SETTINGS_PRINT_SYNTAX_HIGHLIGHTING, syntax);
-	g_settings_set_boolean (job->priv->print_settings, GEDIT_SETTINGS_PRINT_HEADER,
+	g_settings_set_boolean (job->priv->gsettings,
+				GEDIT_SETTINGS_PRINT_SYNTAX_HIGHLIGHTING,
+				syntax);
+
+	g_settings_set_boolean (job->priv->gsettings,
+				GEDIT_SETTINGS_PRINT_HEADER,
 				page_header);
-	g_settings_set_string (job->priv->print_settings, GEDIT_SETTINGS_PRINT_FONT_BODY_PANGO,
-			       body);
-	g_settings_set_string (job->priv->print_settings, GEDIT_SETTINGS_PRINT_FONT_HEADER_PANGO,
-			       header);
-	g_settings_set_string (job->priv->print_settings, GEDIT_SETTINGS_PRINT_FONT_NUMBERS_PANGO,
-			       numbers);
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (job->priv->line_numbers_checkbutton)))
+	g_settings_set_string (job->priv->gsettings,
+			       GEDIT_SETTINGS_PRINT_FONT_BODY_PANGO,
+			       body_font);
+
+	g_settings_set_string (job->priv->gsettings,
+			       GEDIT_SETTINGS_PRINT_FONT_HEADER_PANGO,
+			       header_font);
+
+	g_settings_set_string (job->priv->gsettings,
+			       GEDIT_SETTINGS_PRINT_FONT_NUMBERS_PANGO,
+			       numbers_font);
+
+	if (gtk_toggle_button_get_active (job->priv->line_numbers_checkbutton))
 	{
-		g_settings_set (job->priv->print_settings,
+		gint num;
+
+		num = gtk_spin_button_get_value_as_int (job->priv->line_numbers_spinbutton);
+
+		g_settings_set (job->priv->gsettings,
 				GEDIT_SETTINGS_PRINT_LINE_NUMBERS,
-			"u", MAX (1, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (job->priv->line_numbers_spinbutton))));
+				"u", MAX (1, num));
 	}
 	else
 	{
-		g_settings_set (job->priv->print_settings,
-				GEDIT_SETTINGS_PRINT_LINE_NUMBERS, "u", 0);
+		g_settings_set (job->priv->gsettings,
+				GEDIT_SETTINGS_PRINT_LINE_NUMBERS,
+				"u", 0);
 	}
 
-	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (job->priv->text_wrapping_checkbutton)))
+	if (gtk_toggle_button_get_active (job->priv->text_wrapping_checkbutton))
 	{
-		wrap_mode = GTK_WRAP_NONE;
-	}
-	else
-	{
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (job->priv->do_not_split_checkbutton)))
+		if (gtk_toggle_button_get_active (job->priv->do_not_split_checkbutton))
 		{
 			wrap_mode = GTK_WRAP_WORD;
 		}
@@ -491,12 +415,60 @@ custom_widget_apply_cb (GtkPrintOperation *operation,
 			wrap_mode = GTK_WRAP_CHAR;
 		}
 	}
+	else
+	{
+		wrap_mode = GTK_WRAP_NONE;
+	}
 
-	g_settings_set_enum (job->priv->print_settings,
+	g_settings_set_enum (job->priv->gsettings,
 			     GEDIT_SETTINGS_PRINT_WRAP_MODE,
 			     wrap_mode);
 }
 
+static void
+preview_ready (GtkPrintOperationPreview *gtk_preview,
+	       GtkPrintContext          *context,
+	       GeditPrintJob            *job)
+{
+	job->priv->is_preview = TRUE;
+
+	g_signal_emit (job, print_job_signals[SHOW_PREVIEW], 0, job->priv->preview);
+
+	g_clear_object (&job->priv->preview);
+}
+
+static void
+preview_destroyed (GtkWidget                *preview,
+		   GtkPrintOperationPreview *gtk_preview)
+{
+	gtk_print_operation_preview_end_preview (gtk_preview);
+}
+
+static gboolean
+preview_cb (GtkPrintOperation        *op,
+	    GtkPrintOperationPreview *gtk_preview,
+	    GtkPrintContext          *context,
+	    GtkWindow                *parent,
+	    GeditPrintJob            *job)
+{
+	g_clear_object (&job->priv->preview);
+	job->priv->preview = gedit_print_preview_new (op, gtk_preview, context);
+	g_object_ref_sink (job->priv->preview);
+
+	g_signal_connect_after (gtk_preview,
+			        "ready",
+				G_CALLBACK (preview_ready),
+				job);
+
+	/* FIXME: should this go in the preview widget itself? */
+	g_signal_connect (job->priv->preview,
+			  "destroy",
+			  G_CALLBACK (preview_destroyed),
+			  gtk_preview);
+
+	return TRUE;
+}
+
 static void
 create_compositor (GeditPrintJob *job)
 {
@@ -507,50 +479,58 @@ create_compositor (GeditPrintJob *job)
 	GtkWrapMode wrap_mode;
 	guint print_line_numbers;
 	gboolean print_header;
+	guint tab_width;
 	gdouble margin;
 
-	/* Create and initialize print compositor */
-	print_font_body = g_settings_get_string (job->priv->print_settings,
+	print_font_body = g_settings_get_string (job->priv->gsettings,
 						 GEDIT_SETTINGS_PRINT_FONT_BODY_PANGO);
-	print_font_header = g_settings_get_string (job->priv->print_settings,
+
+	print_font_header = g_settings_get_string (job->priv->gsettings,
 						   GEDIT_SETTINGS_PRINT_FONT_HEADER_PANGO);
-	print_font_numbers = g_settings_get_string (job->priv->print_settings,
+
+	print_font_numbers = g_settings_get_string (job->priv->gsettings,
 						    GEDIT_SETTINGS_PRINT_FONT_NUMBERS_PANGO);
-	syntax_hl = g_settings_get_boolean (job->priv->print_settings,
-					    GEDIT_SETTINGS_PRINT_SYNTAX_HIGHLIGHTING);
-	g_settings_get (job->priv->print_settings, GEDIT_SETTINGS_PRINT_LINE_NUMBERS,
+
+	g_settings_get (job->priv->gsettings, GEDIT_SETTINGS_PRINT_LINE_NUMBERS,
 			"u", &print_line_numbers);
-	print_header = g_settings_get_boolean (job->priv->print_settings,
+
+	print_header = g_settings_get_boolean (job->priv->gsettings,
 					       GEDIT_SETTINGS_PRINT_HEADER);
 
-	wrap_mode = g_settings_get_enum (job->priv->print_settings,
+	wrap_mode = g_settings_get_enum (job->priv->gsettings,
 					 GEDIT_SETTINGS_PRINT_WRAP_MODE);
 
-	job->priv->compositor = GTK_SOURCE_PRINT_COMPOSITOR (
-					g_object_new (GTK_SOURCE_TYPE_PRINT_COMPOSITOR,
-						     "buffer", GTK_SOURCE_BUFFER (job->priv->doc),
-						     "tab-width", gtk_source_view_get_tab_width (GTK_SOURCE_VIEW (job->priv->view)),
-						     "highlight-syntax", gtk_source_buffer_get_highlight_syntax (GTK_SOURCE_BUFFER (job->priv->doc)) &&
-									 syntax_hl,
-						     "wrap-mode", wrap_mode,
-						     "print-line-numbers", print_line_numbers,
-						     "print-header", print_header,
-						     "print-footer", FALSE,
-						     "body-font-name", print_font_body,
-						     "line-numbers-font-name", print_font_numbers,
-						     "header-font-name", print_font_header,
-						     NULL));
+	syntax_hl = g_settings_get_boolean (job->priv->gsettings,
+					    GEDIT_SETTINGS_PRINT_SYNTAX_HIGHLIGHTING);
 
-	margin = g_settings_get_double (job->priv->print_settings, GEDIT_SETTINGS_PRINT_MARGIN_LEFT);
+	syntax_hl &= gtk_source_buffer_get_highlight_syntax (GTK_SOURCE_BUFFER (job->priv->doc));
+
+	tab_width = gtk_source_view_get_tab_width (GTK_SOURCE_VIEW (job->priv->view));
+
+	job->priv->compositor = GTK_SOURCE_PRINT_COMPOSITOR (
+		g_object_new (GTK_SOURCE_TYPE_PRINT_COMPOSITOR,
+			      "buffer", GTK_SOURCE_BUFFER (job->priv->doc),
+			      "tab-width", tab_width,
+			      "highlight-syntax", syntax_hl,
+			      "wrap-mode", wrap_mode,
+			      "print-line-numbers", print_line_numbers,
+			      "print-header", print_header,
+			      "print-footer", FALSE,
+			      "body-font-name", print_font_body,
+			      "line-numbers-font-name", print_font_numbers,
+			      "header-font-name", print_font_header,
+			      NULL));
+
+	margin = g_settings_get_double (job->priv->gsettings, GEDIT_SETTINGS_PRINT_MARGIN_LEFT);
 	gtk_source_print_compositor_set_left_margin (job->priv->compositor, margin, GTK_UNIT_MM);
 
-	margin = g_settings_get_double (job->priv->print_settings, GEDIT_SETTINGS_PRINT_MARGIN_TOP);
+	margin = g_settings_get_double (job->priv->gsettings, GEDIT_SETTINGS_PRINT_MARGIN_TOP);
 	gtk_source_print_compositor_set_top_margin (job->priv->compositor, margin, GTK_UNIT_MM);
 
-	margin = g_settings_get_double (job->priv->print_settings, GEDIT_SETTINGS_PRINT_MARGIN_RIGHT);
+	margin = g_settings_get_double (job->priv->gsettings, GEDIT_SETTINGS_PRINT_MARGIN_RIGHT);
 	gtk_source_print_compositor_set_right_margin (job->priv->compositor, margin, GTK_UNIT_MM);
 
-	margin = g_settings_get_double (job->priv->print_settings, GEDIT_SETTINGS_PRINT_MARGIN_BOTTOM);
+	margin = g_settings_get_double (job->priv->gsettings, GEDIT_SETTINGS_PRINT_MARGIN_BOTTOM);
 	gtk_source_print_compositor_set_bottom_margin (job->priv->compositor, margin, GTK_UNIT_MM);
 
 	if (print_header)
@@ -590,51 +570,12 @@ begin_print_cb (GtkPrintOperation *operation,
 {
 	create_compositor (job);
 
-	job->priv->status = GEDIT_PRINT_JOB_STATUS_PAGINATING;
-
 	job->priv->progress = 0.0;
 
-	g_signal_emit (job, print_job_signals[PRINTING], 0, job->priv->status);
-}
-
-static void
-preview_ready (GtkPrintOperationPreview *gtk_preview,
-	       GtkPrintContext          *context,
-	       GeditPrintJob            *job)
-{
-	job->priv->is_preview = TRUE;
-
-	g_signal_emit (job, print_job_signals[SHOW_PREVIEW], 0, job->priv->preview);
-}
-
-static void
-preview_destroyed (GtkWidget                *preview,
-		   GtkPrintOperationPreview *gtk_preview)
-{
-	gtk_print_operation_preview_end_preview (gtk_preview);
-}
-
-static gboolean
-preview_cb (GtkPrintOperation        *op,
-	    GtkPrintOperationPreview *gtk_preview,
-	    GtkPrintContext          *context,
-	    GtkWindow                *parent,
-	    GeditPrintJob            *job)
-{
-	job->priv->preview = gedit_print_preview_new (op, gtk_preview, context);
-
-	g_signal_connect_after (gtk_preview,
-			        "ready",
-				G_CALLBACK (preview_ready),
-				job);
-
-	/* FIXME: should this go in the preview widget itself? */
-	g_signal_connect (job->priv->preview,
-			  "destroy",
-			  G_CALLBACK (preview_destroyed),
-			  gtk_preview);
-
-	return TRUE;
+	g_signal_emit (job,
+		       print_job_signals[PRINTING],
+		       0,
+		       GEDIT_PRINT_JOB_STATUS_PAGINATING);
 }
 
 static gboolean
@@ -642,13 +583,11 @@ paginate_cb (GtkPrintOperation *operation,
 	     GtkPrintContext   *context,
 	     GeditPrintJob     *job)
 {
-	gboolean res;
+	gboolean finished;
 
-	job->priv->status = GEDIT_PRINT_JOB_STATUS_PAGINATING;
+	finished = gtk_source_print_compositor_paginate (job->priv->compositor, context);
 
-	res = gtk_source_print_compositor_paginate (job->priv->compositor, context);
-
-	if (res)
+	if (finished)
 	{
 		gint n_pages;
 
@@ -659,13 +598,19 @@ paginate_cb (GtkPrintOperation *operation,
 	job->priv->progress = gtk_source_print_compositor_get_pagination_progress (job->priv->compositor);
 
 	/* When previewing, the progress is just for pagination, when printing
-	 * it's split between pagination and rendering */
+	 * it's split between pagination and rendering.
+	 */
 	if (!job->priv->is_preview)
+	{
 		job->priv->progress /= 2.0;
+	}
 
-	g_signal_emit (job, print_job_signals[PRINTING], 0, job->priv->status);
+	g_signal_emit (job,
+		       print_job_signals[PRINTING],
+		       0,
+		       GEDIT_PRINT_JOB_STATUS_PAGINATING);
 
-	return res;
+	return finished;
 }
 
 static void
@@ -677,22 +622,23 @@ draw_page_cb (GtkPrintOperation *operation,
 	gint n_pages;
 
 	/* In preview, pages are drawn on the fly, so rendering is
-	 * not part of the progress */
+	 * not part of the progress.
+	 */
 	if (!job->priv->is_preview)
 	{
-		g_free (job->priv->status_string);
-
 		n_pages = gtk_source_print_compositor_get_n_pages (job->priv->compositor);
 
-		job->priv->status = GEDIT_PRINT_JOB_STATUS_DRAWING;
-
-		job->priv->status_string = g_strdup_printf ("Rendering page %d of %d...",
+		g_free (job->priv->status_string);
+		job->priv->status_string = g_strdup_printf (_("Rendering page %d of %d..."),
 							    page_nr + 1,
 							    n_pages);
 
 		job->priv->progress = page_nr / (2.0 * n_pages) + 0.5;
 
-		g_signal_emit (job, print_job_signals[PRINTING], 0, job->priv->status);
+		g_signal_emit (job,
+			       print_job_signals[PRINTING],
+			       0,
+			       GEDIT_PRINT_JOB_STATUS_DRAWING);
 	}
 
 	gtk_source_print_compositor_draw_page (job->priv->compositor, context, page_nr);
@@ -703,11 +649,7 @@ end_print_cb (GtkPrintOperation *operation,
 	      GtkPrintContext   *context,
 	      GeditPrintJob     *job)
 {
-	if (job->priv->compositor != NULL)
-	{
-		g_object_unref (job->priv->compositor);
-		job->priv->compositor = NULL;
-	}
+	g_clear_object (&job->priv->compositor);
 }
 
 static void
@@ -737,18 +679,25 @@ done_cb (GtkPrintOperation       *operation,
 			g_return_if_reached ();
 	}
 
-	/* Avoid job is destroyed in the handler of the "done" message */
+	/* Avoid that job is destroyed in the handler of the "done" message. */
 	g_object_ref (job);
-
 	g_signal_emit (job, print_job_signals[DONE], 0, print_result, error);
-
-	g_object_unref (operation);
-	job->priv->operation = NULL;
-
 	g_object_unref (job);
 }
 
-/* Note that gedit_print_job_print can can only be called once on a given GeditPrintJob */
+GeditPrintJob *
+gedit_print_job_new (GeditView *view)
+{
+	g_return_val_if_fail (GEDIT_IS_VIEW (view), NULL);
+
+	return g_object_new (GEDIT_TYPE_PRINT_JOB,
+			     "view", view,
+			     NULL);
+}
+
+/* Note that gedit_print_job_print() can only be called once on a given
+ * GeditPrintJob.
+ */
 GtkPrintOperationResult
 gedit_print_job_print (GeditPrintJob            *job,
 		       GtkPrintOperationAction   action,
@@ -757,103 +706,86 @@ gedit_print_job_print (GeditPrintJob            *job,
 		       GtkWindow                *parent,
 		       GError                  **error)
 {
-	GeditPrintJobPrivate *priv;
 	gchar *job_name;
 
+	g_return_val_if_fail (job->priv->operation == NULL, GTK_PRINT_OPERATION_RESULT_ERROR);
 	g_return_val_if_fail (job->priv->compositor == NULL, GTK_PRINT_OPERATION_RESULT_ERROR);
 
-	priv = job->priv;
-
-	/* Check if we are previewing */
-	priv->is_preview = (action == GTK_PRINT_OPERATION_ACTION_PREVIEW);
-
-	/* Create print operation */
 	job->priv->operation = gtk_print_operation_new ();
 
-	if (settings)
-		gtk_print_operation_set_print_settings (priv->operation,
+	job->priv->is_preview = action == GTK_PRINT_OPERATION_ACTION_PREVIEW;
+
+	if (settings != NULL)
+	{
+		gtk_print_operation_set_print_settings (job->priv->operation,
 							settings);
+	}
 
 	if (page_setup != NULL)
-		gtk_print_operation_set_default_page_setup (priv->operation,
+	{
+		gtk_print_operation_set_default_page_setup (job->priv->operation,
 							    page_setup);
+	}
 
-	job_name = gedit_document_get_short_name_for_display (priv->doc);
-	gtk_print_operation_set_job_name (priv->operation, job_name);
+	job_name = gedit_document_get_short_name_for_display (job->priv->doc);
+	gtk_print_operation_set_job_name (job->priv->operation, job_name);
 	g_free (job_name);
 
-	gtk_print_operation_set_embed_page_setup (priv->operation, TRUE);
+	gtk_print_operation_set_embed_page_setup (job->priv->operation, TRUE);
 
-	gtk_print_operation_set_custom_tab_label (priv->operation,
+	gtk_print_operation_set_custom_tab_label (job->priv->operation,
 						  _("Text Editor"));
 
-	gtk_print_operation_set_allow_async (priv->operation, TRUE);
+	gtk_print_operation_set_allow_async (job->priv->operation, TRUE);
 
-	g_signal_connect (priv->operation,
+	g_signal_connect (job->priv->operation,
 			  "create-custom-widget",
 			  G_CALLBACK (create_custom_widget_cb),
 			  job);
-	g_signal_connect (priv->operation,
+
+	g_signal_connect (job->priv->operation,
 			  "custom-widget-apply",
 			  G_CALLBACK (custom_widget_apply_cb),
 			  job);
-  	g_signal_connect (priv->operation,
-			  "begin-print",
-			  G_CALLBACK (begin_print_cb),
-			  job);
-	g_signal_connect (priv->operation,
+
+	g_signal_connect (job->priv->operation,
 			  "preview",
 			  G_CALLBACK (preview_cb),
 			  job);
-  	g_signal_connect (priv->operation,
+
+	g_signal_connect (job->priv->operation,
+			  "begin-print",
+			  G_CALLBACK (begin_print_cb),
+			  job);
+
+	g_signal_connect (job->priv->operation,
 			  "paginate",
 			  G_CALLBACK (paginate_cb),
 			  job);
-	g_signal_connect (priv->operation,
+
+	g_signal_connect (job->priv->operation,
 			  "draw-page",
 			  G_CALLBACK (draw_page_cb),
 			  job);
-	g_signal_connect (priv->operation,
-			  "end-print",
-			  G_CALLBACK (end_print_cb),
-			  job);
-	g_signal_connect (priv->operation,
-			  "done",
-			  G_CALLBACK (done_cb),
-			  job);
 
-	return gtk_print_operation_run (priv->operation,
+	g_signal_connect_object (job->priv->operation,
+				 "end-print",
+				 G_CALLBACK (end_print_cb),
+				 job,
+				 0);
+
+	g_signal_connect_object (job->priv->operation,
+				 "done",
+				 G_CALLBACK (done_cb),
+				 job,
+				 0);
+
+	return gtk_print_operation_run (job->priv->operation,
 					action,
 					parent,
 					error);
 }
 
-static void
-gedit_print_job_init (GeditPrintJob *job)
-{
-	job->priv = gedit_print_job_get_instance_private (job);
-
-	job->priv->print_settings = g_settings_new ("org.gnome.gedit.preferences.print");
-
-	job->priv->status = GEDIT_PRINT_JOB_STATUS_INIT;
-
-	job->priv->status_string = g_strdup (_("Preparing..."));
-}
-
-GeditPrintJob *
-gedit_print_job_new (GeditView *view)
-{
-	GeditPrintJob *job;
-
-	g_return_val_if_fail (GEDIT_IS_VIEW (view), NULL);
-
-	job = GEDIT_PRINT_JOB (g_object_new (GEDIT_TYPE_PRINT_JOB,
-					     "view", view,
-					      NULL));
-
-	return job;
-}
-
 void
 gedit_print_job_cancel (GeditPrintJob *job)
 {
