@@ -572,8 +572,6 @@ search_widget_key_press_event (GtkWidget      *widget,
                                GdkEventKey    *event,
                                GeditViewFrame *frame)
 {
-	guint modifiers = gtk_accelerator_get_default_mod_mask ();
-
 	/* Close window */
 	if (event->keyval == GDK_KEY_Tab)
 	{
@@ -583,35 +581,6 @@ search_widget_key_press_event (GtkWidget      *widget,
 		return GDK_EVENT_STOP;
 	}
 
-	/* Close window and cancel the search */
-	if (event->keyval == GDK_KEY_Escape)
-	{
-		GtkSourceSearchContext *search_context = get_search_context (frame);
-
-		if (frame->priv->search_mode == SEARCH &&
-		    search_context != NULL)
-		{
-			g_clear_object (&frame->priv->search_settings);
-			frame->priv->search_settings = copy_search_settings (frame->priv->old_search_settings);
-
-			gtk_source_search_context_set_settings (search_context,
-								frame->priv->search_settings);
-
-			g_free (frame->priv->search_text);
-			frame->priv->search_text = NULL;
-
-			if (frame->priv->old_search_text != NULL)
-			{
-				frame->priv->search_text = g_strdup (frame->priv->old_search_text);
-			}
-		}
-
-		hide_search_widget (frame, TRUE);
-		gtk_widget_grab_focus (GTK_WIDGET (frame->priv->view));
-
-		return GDK_EVENT_STOP;
-	}
-
 	if (frame->priv->search_mode == GOTO_LINE)
 	{
 		return GDK_EVENT_PROPAGATE;
@@ -626,13 +595,6 @@ search_widget_key_press_event (GtkWidget      *widget,
 		return GDK_EVENT_STOP;
 	}
 
-	if (((event->state & modifiers) == (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) &&
-	    (event->keyval == GDK_KEY_g || event->keyval == GDK_KEY_G))
-	{
-		backward_search (frame);
-		return GDK_EVENT_STOP;
-	}
-
 	/* select next matching iter */
 	if (event->keyval == GDK_KEY_Down || event->keyval == GDK_KEY_KP_Down)
 	{
@@ -640,13 +602,6 @@ search_widget_key_press_event (GtkWidget      *widget,
 		return GDK_EVENT_STOP;
 	}
 
-	if (((event->state & modifiers) == GDK_CONTROL_MASK) &&
-	    (event->keyval == GDK_KEY_g || event->keyval == GDK_KEY_G))
-	{
-		forward_search (frame);
-		return GDK_EVENT_STOP;
-	}
-
 	return GDK_EVENT_PROPAGATE;
 }
 
@@ -905,6 +860,48 @@ setup_popup_menu (GeditViewFrame *frame,
 				  frame);
 }
 
+static void
+search_entry_escaped (GtkSearchEntry *entry,
+                      GeditViewFrame *frame)
+{
+	GtkSourceSearchContext *search_context = get_search_context (frame);
+
+	if (frame->priv->search_mode == SEARCH &&
+	    search_context != NULL)
+	{
+		g_clear_object (&frame->priv->search_settings);
+		frame->priv->search_settings = copy_search_settings (frame->priv->old_search_settings);
+
+		gtk_source_search_context_set_settings (search_context,
+		                                        frame->priv->search_settings);
+
+		g_free (frame->priv->search_text);
+		frame->priv->search_text = NULL;
+
+		if (frame->priv->old_search_text != NULL)
+		{
+			frame->priv->search_text = g_strdup (frame->priv->old_search_text);
+		}
+	}
+
+	hide_search_widget (frame, TRUE);
+	gtk_widget_grab_focus (GTK_WIDGET (frame->priv->view));
+}
+
+static void
+search_entry_previous_match (GtkSearchEntry *entry,
+                             GeditViewFrame *frame)
+{
+	backward_search (frame);
+}
+
+static void
+search_entry_next_match (GtkSearchEntry *entry,
+                         GeditViewFrame *frame)
+{
+	forward_search (frame);
+}
+
 static void
 search_entry_populate_popup (GtkEntry       *entry,
                              GtkMenu        *menu,
@@ -1601,6 +1598,21 @@ gedit_view_frame_init (GeditViewFrame *frame)
 	                  G_CALLBACK (search_entry_insert_text),
 	                  frame);
 
+	g_signal_connect (frame->priv->search_entry,
+	                  "stop-search",
+	                  G_CALLBACK (search_entry_escaped),
+	                  frame);
+
+	g_signal_connect (frame->priv->search_entry,
+	                  "next-match",
+	                  G_CALLBACK (search_entry_next_match),
+	                  frame);
+
+	g_signal_connect (frame->priv->search_entry,
+	                  "previous-match",
+	                  G_CALLBACK (search_entry_previous_match),
+	                  frame);
+
 	frame->priv->search_entry_changed_id =
 		g_signal_connect (frame->priv->search_entry,
 				  "changed",
